import { PluginPass, BabelFile, BabelFileMetadata, BabelFileResult } from '@babel/core';
import * as _babel_types from '@babel/types';
import { Expression } from '@babel/types';
import { NodePath } from '@babel/traverse';

type Macro = (this: Context, ...args: any[]) => Record<string, any> | void;

declare class Context {
    path: NodePath;
    parent: Context | undefined;
    define: Record<string, Context>;
    macros: Record<string, Macro>;
    uid: string;
    also: Set<Context>;
    props: Map<string, any>;
    usedBy: Set<NodePath<_babel_types.Node>>;
    children: Set<Context>;
    condition?: Expression | string;
    alternate?: Context;
    static get(from: NodePath): Context | undefined;
    constructor(path: NodePath, parent?: Context, name?: string);
}

interface Options {
    cssModule?: string;
    macros?: (Record<string, Macro> | false)[];
    define?: Record<string, Context>[];
    polyfill?: string | null;
}
interface State extends PluginPass {
    file: BabelFile & {
        metadata: Preset.MetaData;
    };
}
declare namespace Preset {
    interface Meta extends BabelFileMetadata {
        css: string;
    }
    interface Result extends BabelFileResult {
        metadata: Meta;
        code: string;
    }
    interface MetaData {
        readonly css: string;
        styles: Map<string, Context>;
    }
    export type { Meta, MetaData, Options, Result };
}
declare function Preset(_compiler: any, options?: Preset.Options): any;

export { type Options, Preset, type State, Preset as default };
